using System;
using System.Collections;
using System.Data;
using System.Data.OleDb;
using System.Text;
using System.Xml;
using Gregor.Cgi;

namespace Gregor.Cgi.Media {

internal class CRenderingContext : IDisposable {

    private Hashtable m_HttpQueryParams;
    private Hashtable m_Values;
    private string m_ConnectionString;
    private IDbConnection m_DbConnection;
    private ArrayList m_ExtraDbConnections;
    private int m_Level;

    public CRenderingContext(Hashtable httpQueryParams, string sConnectionString){
        m_HttpQueryParams = httpQueryParams;
        m_ConnectionString = sConnectionString;
        m_ExtraDbConnections = new ArrayList();
        m_Values = new Hashtable();
    }

    public void Dispose(){
        if(m_DbConnection != null && m_DbConnection.State != ConnectionState.Closed){
            m_DbConnection.Close();
            m_DbConnection = null;
        }
        foreach(IDbConnection con in m_ExtraDbConnections){
            if(con.State != ConnectionState.Closed){
                con.Close();
            }
        }
        m_ExtraDbConnections.Clear();
    }

    public Hashtable HttpQueryParams{
        get{
            return m_HttpQueryParams;
        }
    }

    public string ConnectionString{
        get{
            return m_ConnectionString;
        }
    }

    public IDbConnection DbConnection{
        get{
            if(m_DbConnection == null){
                m_DbConnection = new OleDbConnection(m_ConnectionString);
                m_DbConnection.Open();
            }
            return m_DbConnection;
        }
    }

    public int Level{
        get{
            return m_Level;
        }
        set{
            m_Level = value;
        }
    }

    public IDbCommand CreateDbCommand(){
        return this.CreateDbCommand(false);
    }

    public IDbCommand CreateDbCommand(bool fNewConnection){
        if(fNewConnection){
            IDbConnection con = new OleDbConnection(m_ConnectionString);
            con.Open();
            return con.CreateCommand();
        }else{
            return this.DbConnection.CreateCommand();
        }
    }

    public void SetValue(string sName, object oValue){
        m_Values[sName] = oValue;
    }

    public object GetValue(string sName){
        return m_Values[sName];
    }

} // class CRenderingContext

internal interface INamedValue {

    string Name{get;}
    string GetValue();

} // interface INamedValue

internal interface IRenderable {

    string Renderer{get;}
    IRenderer GetRenderer();

} // interface IRenderable

internal interface IRenderer {

    void Render(CRenderingContext context);

} // interface IRenderer

internal abstract class CRenderableBase : IRenderable {

    private string m_Name;
    private string m_Title;
    private string m_Renderer;

    protected CRenderableBase(XmlElement xe){
        m_Name = xe.Attributes["name"].Value;
        m_Title = xe.Attributes["title"].Value;
        m_Renderer = xe.Attributes["renderer"].Value;
    }

    public string Name{
        get{
            return m_Name;
        }
    }

    public string Title{
        get{
            return m_Title;
        }
    }

    public string Renderer{
        get{
            return m_Renderer;
        }
    }

    public virtual IRenderer GetRenderer(){
        Type tp = Type.GetType(m_Renderer, true, true);
        object[] args = new object[]{this};
        object obj = Activator.CreateInstance(tp, args);
        return (IRenderer) obj;
    }

} // class CRenderableBase

} // namespace Gregor.Cgi.Media