using System;
using System.Collections;
using System.Data;
using System.Data.OleDb;
using System.Text;
using System.Xml;
using Gregor.Cgi;

namespace Gregor.Cgi.Media {

internal class CInputInfo : CRenderableBase, INamedValue {

    private string m_SqlLookupText;
    private string m_SqlDefaultText;

    public CInputInfo(XmlElement xe) : base(xe) {
        XmlAttribute xa = xe.Attributes["sql-lookup-text"];
        if(xa != null){
            m_SqlLookupText = xa.Value;
        }
        xa = xe.Attributes["sql-default-text"];
        if(xa != null){
            m_SqlDefaultText = xa.Value;
        }
    }

    public string SqlLookupText{
        get{
            return m_SqlLookupText;
        }
    }

    public string SqlDefaultText{
        get{
            return m_SqlDefaultText;
        }
    }

    public string GetValue(){
        string s = CEnv.GetQueryParam(this.Name);
        if(s != null){
            s = CHelper.HexUnEscape(s);
        }
        return s;
    }

} // class CInputInfo

internal class CInputRenderer : IRenderer {

    private CInputInfo m_Input;

    public CInputRenderer(CInputInfo input){
        m_Input = input;
    }

    public CInputInfo Input{
        get{
            return m_Input;
        }
    }

    public virtual void Render(CRenderingContext context){
        COut.WriteHtmlTableRowOpener();
        COut.WriteHtmlTableDataCell(this.Input.Title, "Left");
        string sValue = null;
        object oValue = context.GetValue(this.Input.Name);
        if(oValue != null){
            sValue = oValue.ToString();
        }else if(this.Input.SqlDefaultText != null){
            sValue = CHelper.GetScalarValue(context, this.Input.SqlDefaultText);
        }else{
            sValue = this.Input.GetValue();
        }
        object oElem = null;
        if(this.Input.SqlLookupText != null){
            oElem = CHelper.CreateLookupInput(this.Input.Name, sValue, context, this.Input.SqlLookupText, null);
        }else{
            oElem = CreateInputElement(this.Input.Name, sValue);
        }
        COut.WriteHtmlTableDataCell(oElem);
        COut.WriteHtmlTableRowCloser();
    }

    protected virtual string CreateInputElement(string sName, string sValue){
        return CHelper.CreateInput(sName, sValue);
    }

} // class CInputRenderer

internal class CLargeInputRenderer : CInputRenderer {

    public CLargeInputRenderer(CInputInfo input) : base(input) {
        // OK
    }

    protected override string CreateInputElement(string sName, string sValue){
        return CHelper.CreateLargeInput(sName);
    }

} // class CLargeInputRenderer

} // namespace Gregor.Cgi.Media