using System;
using System.Collections;
using System.Data;
using System.Data.OleDb;
using System.Text;
using System.Xml;
using Gregor.Cgi;

namespace Gregor.Cgi.Media {

internal class CPageInfo : CRenderableBase {

    private ArrayList m_Tables;
    private ArrayList m_Sheets;
    private ArrayList m_Links;

    public CPageInfo(XmlElement xe): base(xe){
        m_Tables = new ArrayList();
        m_Sheets = new ArrayList();
        m_Links = new ArrayList();
        foreach(XmlNode xnChild in xe){
            if(xnChild is XmlElement){
                XmlElement xeChild = (XmlElement) xnChild;
                switch(xeChild.Name){
                    case "table":{
                        CTableInfo table = new CTableInfo(xeChild);
                        m_Tables.Add(table);
                        break;
                    }
                    case "sheet":{
                        CSheetInfo sheet = new CSheetInfo(xeChild);
                        m_Sheets.Add(sheet);
                        break;
                    }
                    case "link":{
                        CLinkInfo link = new CLinkInfo(xeChild);
                        m_Links.Add(link);
                        break;
                    }
                }
            }
        }
    }

    public ArrayList Tables{
        get{
            return m_Tables;
        }
    }

    public ArrayList Sheets{
        get{
            return m_Sheets;
        }
    }

    public ArrayList Links{
        get{
            return m_Links;
        }
    }

} // class CPageInfo

internal abstract class CPageRendererBase : IRenderer {

    private CPageInfo m_Page;

    protected CPageRendererBase(CPageInfo page){
        m_Page = page;
    }

    public CPageInfo Page{
        get{
            return m_Page;
        }
    }

    public abstract void Render(CRenderingContext context);

} // class CPageRendererBase

internal class CDefaultPageRenderer : CPageRendererBase {

    public CDefaultPageRenderer(CPageInfo page) : base(page){
    }

    public override void Render(CRenderingContext context){
        context.Level++;
        COut.WriteHtmlParagraph("[" + this.Page.Title + "]", "Seitenkopf");
        COut.WriteHtmlHeading(context.Level, "Page: " + this.Page.Title);
        foreach(CSheetInfo sheet in this.Page.Sheets){
            IRenderer renderer = sheet.GetRenderer();
            renderer.Render(context);
        }
        foreach(CTableInfo table in this.Page.Tables){
            IRenderer renderer = table.GetRenderer();
            renderer.Render(context);
        }
        if(this.Page.Links.Count > 0){
            COut.WriteHtmlDivider();
            COut.WriteHtmlParagraphOpener();
            bool fAfterFirst = false;
            foreach(CLinkInfo link in this.Page.Links){
                if(fAfterFirst){
                    COut.W(" | ");
                }
                IRenderer renderer = link.GetRenderer();
                renderer.Render(context);
                fAfterFirst = true;
            }
            COut.WriteHtmlParagraphCloser();
        }
        context.Level--;
    }

} // class CDefaultPageRenderer

} // namespace Gregor.Cgi.Media