using System;
using System.Collections;
using System.Data;
using System.Data.OleDb;
using System.Text;
using System.Xml;
using Gregor.Cgi;

namespace Gregor.Cgi.Media {

internal abstract class CSqlRenderableBase : CRenderableBase {

    private string m_SqlText;
    private ArrayList m_Keys;

    protected CSqlRenderableBase(XmlElement xe) : base(xe){
        XmlAttribute xaSql = xe.Attributes["sql-text"];
        if(xaSql != null){
            m_SqlText = xaSql.Value;
        }else{
            m_SqlText = string.Empty;
        }
        m_Keys = new ArrayList();
        foreach(XmlNode xnChild in xe.ChildNodes){
            if(xnChild is XmlElement){
                XmlElement xeChild = (XmlElement) xnChild;
                switch(xeChild.Name){
                    case "key":
                        CKeyInfo key = new CKeyInfo(xeChild);
                        m_Keys.Add(key);
                        break;
                }
            }
        }
    }

    public string SqlText{
        get{
            return m_SqlText;
        }
    }

    public ArrayList Keys{
        get{
            return m_Keys;
        }
    }

} // class CSqlRenderableBase

internal abstract class CSqlRendererBase : IRenderer {

    private CSqlRenderableBase m_Renderable;

    protected CSqlRendererBase(CSqlRenderableBase renderable){
        m_Renderable = renderable;
    }

    public CSqlRenderableBase Renderable{
        get{
            return m_Renderable;
        }
    }

    public virtual void Render(CRenderingContext context){
        context.Level++;
        IDbCommand cmd = this.CreateCommand(context);
        this.DoRender(context, cmd);
        context.Level--;
    }

    protected virtual IDbCommand CreateCommand(CRenderingContext context){
        return CHelper.CreateSelectCommand(context, this.Renderable.SqlText, this.Renderable.Keys, false);
    }

    protected abstract void DoRender(CRenderingContext context, IDbCommand cmd);

} // class CSqlRendererBase

} // namespace Gregor.Cgi.Media