// <clown> csc.exe /debug+ /r:System.dll,System.Data.dll Gregor.Cgi.Media.cs Gregor.Cgi.Media.Base.cs Gregor.Cgi.Media.SqlBase.cs Gregor.Cgi.Media.Page.cs Gregor.Cgi.Media.Table.cs Gregor.Cgi.Media.Sheet.cs Gregor.Cgi.Media.Input.cs Gregor.Cgi.Media.Key.cs Gregor.Cgi.Media.Link.cs Gregor.Cgi.cs </clown>

using System;
using System.Collections;
using System.Data;
using System.Data.OleDb;
using System.Text;
using System.Xml;
using Gregor.Cgi;

namespace Gregor.Cgi.Media {

public class C {

    public const string EXE_NAME = "Gregor.Cgi.Media.exe";

    public static void Main(string[] args){

        try{

            string sTrace = CConfig.GetParam("trace");
            if(sTrace != null && sTrace.Equals("1")){
                COut.TraceOn = true;
            }

            COut.WriteResponseHeader();

            C.WriteDocHeader();

            Hashtable httpQueryParams = CEnv.GetQueryParams();

            if(false == CEnv.HasQueryString){

                WriteIntro(httpQueryParams);

            }else{

                string sSpecial = CEnv.GetQueryParam("special");
                if(sSpecial != null){
                    WriteSpecial(sSpecial, httpQueryParams);
                }

                string sPageName = CEnv.GetQueryParam("page");
                if(sPageName != null){
                    WritePage(sPageName, httpQueryParams);
                }

                string sInsert = CEnv.GetQueryParam("insert");
                if(sInsert != null){
                    ProcessInsert(sInsert, httpQueryParams);
                }

                string sUpdate = CEnv.GetQueryParam("update");
                if(sUpdate != null){
                    ProcessUpdate(sUpdate, httpQueryParams);
                }
            }

        }catch(Exception ex){
            COut.WriteHtmlErrorMessage(ex);
        }finally{
            COut.WriteDocFooter();
        }

    }

    private static void WriteDocHeader(){
        string sTitle = "Media";
        if(CEnv.HasQueryString){
            string sPageName = CEnv.GetQueryParam("page");
            if(sPageName != null){
                CPageInfo page = GetPage(sPageName);
                sTitle = "Media." + page.Title;
            }
        }
        string sStyleSheet = CConfig.GetParam("css-stylesheet");
        COut.WriteDocHeader(sTitle, sStyleSheet);
    }

    private static void WriteIntro(Hashtable httpQueryParams){
        WritePage("page-intro", httpQueryParams);
    }

    private static void WriteSpecial(string sSpecial, Hashtable httpQueryParams){
        switch(sSpecial){
            case "config":
                WriteConfig(httpQueryParams);
                break;
            default:
                COut.WriteHtmlErrorMessage("Not supported: '" + sSpecial + "'.");
                break;
        }
    }

    private static void WriteConfig(Hashtable httpQueryParams){

        COut.WriteHtmlParagraph("[Configuration]", "Seitenkopf");
        COut.WriteHtmlHeading(1, "Configuration");

        string sConnect = CConfig.GetParam("connection-string");
        COut.WriteHtmlParagraph("<b>connection-string:</b> " + sConnect);

        ArrayList allPages = GetAllPages();

        foreach(CPageInfo page in allPages){
            COut.WriteHtmlHeading(2, page.Name);
            foreach(CSheetInfo sheet in page.Sheets){
                COut.WriteHtmlParagraph("<b>" + sheet.Name + ":</b> " + sheet.SqlText);
                // keys, inputs ...
            }
            foreach(CTableInfo table in page.Tables){
                COut.WriteHtmlParagraph("<b>" + table.Name + ":</b> " + table.SqlText);
                // keys ...
            }
        }

    }

    private static void WritePage(string sPageName, Hashtable httpQueryParams){
        CPageInfo page = GetPage(sPageName);
        WritePage(page, httpQueryParams);
    }

    private static void WritePage(CPageInfo page, Hashtable httpQueryParams){
        string sConnect = CConfig.GetParam("connection-string");
        CRenderingContext context = new CRenderingContext(httpQueryParams, sConnect);
        try{
            IRenderer renderer = page.GetRenderer();
            renderer.Render(context);
        }finally{
            if(context != null){
                context.Dispose();
            }
        }
    }

    private static ArrayList GetAllPages(){
        ArrayList list = new ArrayList();
        foreach(XmlNode xn in CConfig.GetConfigDoc().DocumentElement){
            if(xn is XmlElement){
                XmlElement xe = (XmlElement) xn;
                if(xe.Name.Equals("page")){
                    CPageInfo page = new CPageInfo(xe);
                    list.Add(page);
                }
            }
        }
        return list;
    }

    internal static CPageInfo GetPage(string sName){
        CPageInfo ret = null;
        foreach(CPageInfo page in GetAllPages()){
            if(page.Name.Equals(sName)){
                ret = page;
                break;
            }
        }
        return ret;
    }

    internal static CSheetInfo GetSheet(string sName){
        foreach(CPageInfo page in GetAllPages()){
            foreach(CSheetInfo sheet in page.Sheets){
                if(sheet.Name.Equals(sName)){
                    return sheet;
                }
            }
        }
        return null;
    }

    private static void ProcessUpdate(string sUpdate, Hashtable httpQueryParams){
        string sConnect = CConfig.GetParam("connection-string");
        CRenderingContext context = new CRenderingContext(httpQueryParams, sConnect);
        try{
            PerformUpdate(context, sUpdate);
        }finally{
            if(context != null){
                context.Dispose();
            }
        }
        CSheetInfo sheet = C.GetSheet(sUpdate);
        if(sheet.NextPage != null){
            WritePage(sheet.NextPage, httpQueryParams);
        }else{
            WriteIntro(httpQueryParams);
        }
    }

    private static void PerformUpdate(CRenderingContext context, string sUpdate){
        IDbCommand cmd = CHelper.CreateUpdateCommand(context);
        int cRows = cmd.ExecuteNonQuery();
        if(1 != cRows){
            throw new Exception("Update '" + sUpdate + "' failed (" + cRows + " rows affected).");
        }
    }

    private static void ProcessInsert(string sInsert, Hashtable httpQueryParams){
        string sConnect = CConfig.GetParam("connection-string");
        CRenderingContext context = new CRenderingContext(httpQueryParams, sConnect);
        try{
            PerformInsert(context, sInsert);
        }finally{
            if(context != null){
                context.Dispose();
            }
        }
        CSheetInfo sheet = C.GetSheet(sInsert);
        if(sheet.NextPage != null){
            WritePage(sheet.NextPage, httpQueryParams);
        }else{
            WriteIntro(httpQueryParams);
        }
    }

    private static void PerformInsert(CRenderingContext context, string sInsert){
        IDbCommand cmd = CHelper.CreateInsertCommand(context);
        int cRows = cmd.ExecuteNonQuery();
        if(1 != cRows){
            throw new Exception("Insert '" + sInsert + "' failed(" + cRows + " rows affected).");
        }
    }

} // class C

internal class CHelper {

    internal static string CreatePageLink(object oFriendlyName, string sPageName, string sColName, string sColValue){
        oFriendlyName = FormatValue(oFriendlyName);
        return "<a href=\"" + C.EXE_NAME + "?page=" + sPageName + ";" + sColName + "=" + sColValue + "\">" + oFriendlyName + "</a>";
    }

    internal static string CreatePageLink(object oFriendlyName, string sPageName){
        oFriendlyName = FormatValue(oFriendlyName);
        return "<a href=\"" + C.EXE_NAME + "?page=" + sPageName + ";\">" + oFriendlyName + "</a>";
    }

    internal static string CreateSpecialLink(object oFriendlyName, string sSpecial){
        oFriendlyName = FormatValue(oFriendlyName);
        return "<a href=\"" + C.EXE_NAME + "?special=" + sSpecial + ";\">" + oFriendlyName + "</a>";
    }

    internal static string FormatValue(object obj){
        string sRet = string.Empty;
        if(Convert.IsDBNull(obj)){
            sRet = "&nbsp;";
        }else if(obj is string){
            sRet = obj.ToString();
        }else if(obj is DateTime){
            sRet = ((DateTime)obj).ToShortDateString(); // ... what about timespans?
        }else{
            sRet = obj.ToString();
        }
        sRet = sRet.Replace(Environment.NewLine, "<br>"); // ...
        return sRet;
    }

    internal static bool IsNumeric(string s){
        bool fRet = (s.Length > 0);
        foreach(char ch in s){
            fRet = fRet && char.IsDigit(ch);
        }
        return fRet;
    }

    internal static string CreateInput(string sName){
        return CreateInput(sName, null);
    }

    internal static string CreateInput(string sName, string sDefaultValue){
        if(null == sDefaultValue){
            sDefaultValue = string.Empty;
        }
        return "<input type=\"text\" name=\"" + sName + "\" value=\"" + sDefaultValue + "\" />";
    }

    internal static string CreateLargeInput(string sName){
        return "<textarea name=\"" + sName + "\" value=\"\" cols=\"80\" rows=\"25\"></textarea>";
    }

    internal static string CreateLookupInput(string sName, string sDefaultValue, CRenderingContext context, string sSqlLookupText, ArrayList keys){
        StringBuilder sb = new StringBuilder();
        sb.Append("<select name=\"" + sName + "\">");
        if(null == sDefaultValue){
            sDefaultValue = string.Empty;
        }
        IDbCommand cmd = CreateSelectCommand(context, sSqlLookupText, keys, true);
        IDataReader reader = null;
        try{
            reader = cmd.ExecuteReader();
            while(reader.Read()){
                string sValue = string.Empty;
                string sText = string.Empty;
                if(reader.FieldCount > 0){
                    sValue = reader.GetValue(0).ToString();
                    if(reader.FieldCount == 1){
                        sText = sValue;
                    }
                }
                bool fAfterFirst = false;
                for(int i = 1; i < reader.FieldCount; i++){
                    if(fAfterFirst){
                        sText += ", ";
                    }
                    sText += reader.GetValue(i).ToString();
                    fAfterFirst = true;
                }
                sb.Append("<option value=\"" + sValue + "\"");
                if(0 == string.Compare(sValue, sDefaultValue)){
                    sb.Append(" selected ");
                }
                sb.Append(">" + sText + "</option>");
            }
        }finally{
            if(reader != null){
                reader.Close();
            }
        }
        sb.Append("</select>");
        return sb.ToString();
    }

    internal static string HexUnEscape(string s){
        StringBuilder sb = new StringBuilder(s.Length);
        int index = 0;
        while(index < s.Length){
            char ch = Uri.HexUnescape(s, ref index);
            sb.Append(ch);
        }
        sb.Replace("+", " ");
        return sb.ToString();
    }

    internal static string GetScalarValue(CRenderingContext context, string sSqlText){
        IDbCommand cmd = context.CreateDbCommand(true);
        cmd.CommandText = sSqlText;
        object obj = cmd.ExecuteScalar();
        return obj.ToString();
    }

    internal static IDbCommand CreateSelectCommand(CRenderingContext context, string sSqlText, ArrayList keys, bool fNewConnection){
        IDbCommand ret = context.CreateDbCommand(fNewConnection);
        ret.CommandText = sSqlText;
        if(keys != null && keys.Count > 0){
            foreach(CKeyInfo key in keys){
                OleDbParameter param = CreateParameter(key);
                ret.Parameters.Add(param);
            }
        }
        COut.Trace(ret.CommandText);
        return ret;
    }

    internal static IDbCommand CreateUpdateCommand(CRenderingContext context){
        return CreateUpdateCommandFromSqlText(context);
    }

    internal static IDbCommand CreateUpdateCommandFromSqlText(CRenderingContext context){
        IDbCommand ret = context.DbConnection.CreateCommand();
        CSheetInfo sheet = C.GetSheet(CEnv.GetQueryParam("update"));
        ret.CommandText = sheet.SqlUpdateText;
        foreach(CInputInfo input in sheet.Inputs){
            OleDbParameter param = CreateParameter(input);
            ret.Parameters.Add(param);
        }
        foreach(CKeyInfo key in sheet.Keys){
            OleDbParameter param = CreateParameter(key);
            ret.Parameters.Add(param);
        }
        COut.Trace(ret.CommandText);       
        return ret;
    }

    internal static IDbCommand CreateInsertCommand(CRenderingContext context){
        return CreateInsertCommandFromSqlText(context);
    }

    internal static IDbCommand CreateInsertCommandFromSqlText(CRenderingContext context){
        IDbCommand ret = context.DbConnection.CreateCommand();
        CSheetInfo sheet = C.GetSheet(CEnv.GetQueryParam("insert"));
        ret.CommandText = sheet.SqlInsertText;
        foreach(CInputInfo input in sheet.Inputs){
            OleDbParameter param = CreateParameter(input);
            ret.Parameters.Add(param);
        }
        COut.Trace(ret.CommandText);
        return ret;
    }

    internal static OleDbParameter CreateParameter(INamedValue nv){
        OleDbParameter ret = null;
        string sKey = nv.Name;
        string sValue = nv.GetValue();
        if(sValue == null || sValue.Length == 0){
            ret = new OleDbParameter(sKey, DBNull.Value);
        }else{
            ret = new OleDbParameter(sKey, sValue);
        }
        COut.Trace(ret.ParameterName + " (" + ret.DbType + "): " + ret.Value);
        return ret;
    }

} // class CHelper

} // namespace Gregor.Cgi.Media