using System;
using System.Collections;
using System.Data;
using System.Data.OleDb;
using System.Xml;

namespace Gregor.Cgi {

public class COut {

    private static bool s_TraceOn;

    public static bool TraceOn{
        get{
            return s_TraceOn;
        }
        set{
            s_TraceOn = value;
        }
    }

    public static void WriteResponseHeader(){
        W("content-type: text/html");
        W(string.Empty);
    }

    public static void WriteDocHeader(string sPageTitle, string sStyleSheet){
        W("<html>");
        W("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">");
        W("<link rel=\"stylesheet\" href=\"" + sStyleSheet + "\" type=\"text/css\" media=\"screen\" />");
        W("<head>");
        W("<title>" + sPageTitle + "</title>");
        W("</head>");
        W("<body>");
    }

    public static void WriteDocFooter(){
        W("</body>");
        W("</html>");
    }

    public static void WriteHtmlHeading(int level, string sText){
        W("<h" + level + ">" + sText + "</h" + level + ">");
    }

    public static void WriteHtmlParagraphOpener(){
        W("<p>");
    }

    public static void WriteHtmlParagraphCloser(){
        W("</p>");
    }

    public static void WriteHtmlParagraph(string sText){
        W("<p>" + sText + "</p>");
    }

    public static void WriteHtmlParagraph(string sText, string sStyleClass){
        W("<p class=\"" + sStyleClass + "\">" + sText + "</p>");
    }

    public static void WriteHtmlDivider(){
        W("<hr/>");
    }

    public static void WriteHtmlErrorMessage(object obj){
        W("<p style=\"color: red; font-family: Courier New;\" >");
        string s = obj.ToString();
        s = s.Replace(Environment.NewLine, "<br>");
        if(obj is OleDbException){
            OleDbException ex = (OleDbException) obj;
            foreach(OleDbError err in ex.Errors){
                W(err.ToString() + "<br>");
            }
        }
        W(s);
        W("</p>");
    }

    public static void WriteHtmlTableOpener(){
        W("<table width=\"100%\" border=\"1\" >");
    }

    public static void WriteHtmlTableCloser(){
        W("</table>");
    }

    public static void WriteHtmlTableHeadOpener(){
        W("<thead>");
    }

    public static void WriteHtmlTableHeadCloser(){
        W("</thead>");
    }

    public static void WriteHtmlTableHeadCell(object content){
        W("<th>" + content.ToString() + "</th>");
    }

    public static void WriteHtmlTableRowOpener(){
        W("<tr>");
    }

    public static void WriteHtmlTableRowCloser(){
        W("</tr>");
    }

    public static void WriteHtmlTableDataCellOpener(){
        W("<td>");
    }

    public static void WriteHtmlTableDataCellCloser(){
        W("</td>");
    }

    public static void WriteHtmlTableDataCell(object content){
        W("<td>" + content.ToString() + "</td>");
    }

    public static void WriteHtmlTableDataCell(object content, string sStyleClass){
        W("<td class=\"" + sStyleClass + "\">" + content.ToString() + "</td>");
    }

    public static void WriteHtmlFormOpener(string sActionBase){
        W("<form action=\"" + sActionBase + "\" method=\"get\">");
    }

    public static void WriteHtmlHiddenField(string sName, string sValue){
        W("<input type=\"hidden\" name=\"" + sName + "\" value=\"" + sValue + "\" />");
    }

    public static void WriteHtmlFormCloser(){
        W("</form>");
    }

    public static void WriteHtmlSubmitButton(){
        WriteHtmlButton("submit");
    }

    public static void WriteHtmlButton(string sType){
        W("<input type=\"" + sType + "\" />");
    }

    public static void WritePre(object o){
        W("<pre>");
        W(o);
        W("</pre>");
    }

    public static void Trace(object o){
        if(s_TraceOn){
            WritePre(o);
        }
    }

    public static void WriteEnvironmentVariables(){
        foreach(DictionaryEntry di in Environment.GetEnvironmentVariables()){
            COut.W(di.Key + ":" + di.Value + "<br>");
        }
    }

    public static void W(object o){
        Console.WriteLine(o);
    }

} // class COut

public class CConfig {

    private static Hashtable s_Params;
    private static XmlDocument s_ConfigDoc;

    public static string GetParam(string sName){
        return GetParam(sName, null);
    }

    public static string GetParam(string sName, string sDefault){
        Hashtable parms = GetParams();
        if(parms.Contains(sName)){
            return (string) parms[sName];
        }else{
            return sDefault;
        }
    }

    public static Hashtable GetParams(){
        if(s_Params == null){
            s_Params = new Hashtable();
            XmlDocument configDoc = GetConfigDoc();
            foreach(XmlNode xn in configDoc.DocumentElement){
                if(xn is XmlElement){
                    XmlElement xe = (XmlElement) xn;
                    if(xe.Name.Equals("param")){
                        string sName = xe.Attributes["name"].Value;
                        string sValue = xe.Attributes["value"].Value;
                        s_Params.Add(sName, sValue);
                    }
                }
            }
        }
        return s_Params;
    }

    public static XmlDocument GetConfigDoc(){
        if(s_ConfigDoc == null){
            s_ConfigDoc = new XmlDocument();
            s_ConfigDoc.Load(GetConfigFileName());
        }
        return s_ConfigDoc;
    }

    private static string GetConfigFileName(){
        string s = AppDomain.CurrentDomain.FriendlyName;
        int i = s.IndexOf(".exe");
        return s.Substring(0, i) + ".Config.xml";
    }

} // class CConfig

public class CEnv {

    private static Hashtable s_QueryParams;

    public static bool HasQueryString{
        get{
            return (null != GetQueryString());
        }
    }

    public static string GetEnvironmentVariable(string sName){
        return Environment.GetEnvironmentVariable(sName);
    }

    public static string GetQueryString(){
        return GetEnvironmentVariable("QUERY_STRING");
    }

    public static Hashtable GetQueryParams(){
        if(s_QueryParams == null){
            string sQuery = GetQueryString();
            if(sQuery != null){
                s_QueryParams = new Hashtable();
                string[] asQueryParams = sQuery.Split(';', '&');
                foreach(string sQueryParam in asQueryParams){
                    string[] asNV = sQueryParam.Split('=');
                    if(asNV.Length == 2){
                        string sName = asNV[0].Trim();
                        string sValue = asNV[1].Trim();
                        s_QueryParams[sName] = sValue;
                    }
                }
            }
        }
        return s_QueryParams;
    }

    public static string GetQueryParam(string sName){
        string sRet = null;
        Hashtable queryParams = GetQueryParams();
        if(queryParams != null){
            sRet = (string) queryParams[sName];
        }
        return sRet;
    }


} // class CEnv

} // namespace Gregor.Cgi